*** check_disk.c.original	Tue Feb 11 16:10:33 2003
--- check_disk.c	Wed Feb 12 11:37:32 2003
***************
*** 44,49 ****
--- 44,61 ----
  int check_disk (int usp, int free_disk);
  void print_help (void);
  void print_usage (void);
+ struct check_list_struct { 
+ 	int last; 
+ 	int w; 
+ 	int c; 
+ 	float wpc; 
+ 	float cpc; 
+ 	char *p; 
+ }; /* check_list_struct */
+ void check_list_destroy ( struct check_list_struct *check_listP);
+ int check_list_add( struct check_list_struct **check_listPP, int w, int c, float wpc, float cpc, char *p); 
+ int check_list_find( struct check_list_struct *check_listP, int *wP,int *cP, float *wpcP,float *cpcP,char *mntp, char *file_systems);
+ struct check_list_struct *w_check_listP=NULL;
  
  const char *progname = "check_disk";
  
***************
*** 55,66 ****
--- 67,81 ----
  char *exclude_device = "";
  int verbose = 0;
  int display_mntp = FALSE;
+ int list_flag=FALSE;
  
  
  int
  main (int argc, char **argv)
  {
+ 	int i=0;
  	int usp = -1;
+ 
  	int total_disk = -1;
  	int used_disk = -1;
  	int free_disk = -1;
***************
*** 74,84 ****
  
  	if (process_arguments (argc, argv) != OK)
  		usage ("Could not parse arguments\n");
  
- 	asprintf (&command_line, "%s %s", DF_COMMAND, path);
- 
  	if (verbose>0)
! 		printf ("%s ==> ", command_line);
  
  	child_process = spopen (command_line);
  	if (child_process == NULL) {
--- 89,108 ----
  
  	if (process_arguments (argc, argv) != OK)
  		usage ("Could not parse arguments\n");
+ 	
+ 	if ( list_flag == FALSE )
+         	asprintf (&command_line, "%s %s", DF_COMMAND, path);
+ 	else {
+ 		/* Construct df command from list of paths or devices */
+ 		i=0;
+         	asprintf (&command_line, "%s", DF_COMMAND);
+ 		do {
+ 			asprintf(&command_line, "%s %s",command_line, w_check_listP[i].p); 
+ 		} while (w_check_listP[i++].last != TRUE );
+ 	}
  
  	if (verbose>0)
! 		printf ("Command: %s\n", command_line);
  
  	child_process = spopen (command_line);
  	if (child_process == NULL) {
***************
*** 108,113 ****
--- 132,146 ----
  				continue;
   			}
  
+ 			/* More than one set of -p so set thresholds from list */
+ 			if ( list_flag==TRUE )
+ 				check_list_find (w_check_listP, 
+ 				  &w_df, &c_df, &w_dfp, &c_dfp, mntp, file_system);
+ 				
+ 			if (verbose>1)
+ 				printf("dev:%s warn:%d crit:%d warn%%=%f crit%%=%f\n",
+ 				  file_system,w_df,c_df,w_dfp,c_dfp);
+ 
  			disk_result = check_disk (usp, free_disk);
  
  			if (strcmp (file_system, "none") == 0)
***************
*** 149,154 ****
--- 182,188 ----
  	else
  		printf ("DISK %s%s\n", state_text (result), output);
  
+ 	check_list_destroy(w_check_listP); 
  	return result;
  }
  
***************
*** 157,162 ****
--- 191,197 ----
  process_arguments (int argc, char **argv)
  {
  	int c;
+ 	int list_row_count=0;
  
  #ifdef HAVE_GETOPT_H
  	int option_index = 0;
***************
*** 172,177 ****
--- 207,213 ----
  		{"mountpoint", no_argument, 0, 'm'},
  		{"exclude_device", required_argument, 0, 'x'},
  		{"quiet", no_argument, 0, 'q'},
+ 		{"clear", no_argument, 0, 'C'},
  
  		{0, 0, 0, 0}
  	};
***************
*** 187,193 ****
  	while (1) {
  #ifdef HAVE_GETOPT_H
  		c =
!  			getopt_long (argc, argv, "+?Vqhvt:c:w:p:x:m", long_options, &option_index);
  #else
   		c = getopt (argc, argv, "+?Vqhvt:c:w:p:x:m");
  #endif
--- 223,229 ----
  	while (1) {
  #ifdef HAVE_GETOPT_H
  		c =
!  			getopt_long (argc, argv, "+?VqhvCt:c:w:p:x:m", long_options, &option_index);
  #else
   		c = getopt (argc, argv, "+?Vqhvt:c:w:p:x:m");
  #endif
***************
*** 237,242 ****
--- 273,285 ----
  				usage ("Timeout Interval must be an integer!\n");
  			}
  		case 'p':									/* path or partition */
+ 			/* Add to check list array the partition and thresholds */
+ 			list_row_count = 
+ 			  check_list_add(&w_check_listP, w_df, c_df, w_dfp, c_dfp, optarg);
+ 			if (list_row_count < 2 )
+ 				list_flag = FALSE;
+ 			else
+ 				list_flag = TRUE;
  			path = optarg;
  			break;
  		case 'v':									/* verbose */
***************
*** 251,256 ****
--- 294,305 ----
   		case 'x':									/* exclude path or partition */
   			exclude_device = optarg;
   			break;
+ 		case 'C':									/* clear thresholds */
+ 			w_df = -1;
+ 			c_df = -1;
+ 			w_dfp = -1.0;
+ 			c_dfp = -1.0;
+ 			break;
  		case 'V':									/* version */
  			print_revision (progname, REVISION);
  			exit (STATE_OK);
***************
*** 348,353 ****
--- 397,404 ----
  		 "    Ignore device (only works if -p unspecified)\n"
  		 " -e, --errors-only\n"
  		 "    Display only devices/mountpoints with errors\n"
+ 		 " -C, --clear\n"
+ 		 "    Clear thresholds\n"
  		 " -v, --verbose\n"
  		 "    Show details for command-line debugging (do not use with nagios server)\n"
  		 " -h, --help\n"
***************
*** 363,366 ****
--- 414,513 ----
  		("Usage: %s -w limit -c limit [-p path | -x device] [-t timeout] [-m] [-e] [--verbose]\n"
  		 "       %s (-h|--help)\n"
  		 "       %s (-V|--version)\n", progname, progname, progname);
+ }
+ 
+ void check_list_destroy (struct check_list_struct *check_listP)
+ {
+ 	int i=0;
+ 
+ 	if (check_listP==NULL)
+ 		return;
+ 	
+ 	/* Free all the strings in the array first */
+ 	for (i=0; check_listP[i].last == FALSE; i++)
+ 		free (check_listP[i].p);
+ 	free (check_listP[i].p);
+ 
+ 	/* Free the array */
+ 	free(check_listP);
+ }
+ 
+ 
+ /* Function to add a item to the check list array */
+ int check_list_add (struct check_list_struct **check_listPP, int w, int c, float wpc, float cpc, char *p)
+ {
+ 	int rows = 0;
+ 	int i;
+ 	int buffer_size = 0;
+ 
+ 	/* Count the number of entries */
+ 	struct check_list_struct *alP;
+ 
+ 	alP = *check_listPP;
+ 
+ 	/* Count the number of entries */
+ 	if (alP == NULL)
+ 		/* No entries in the list yet */
+ 		rows = 0;
+ 	else
+ 	{
+ 		for (i = 0; alP[i].last != TRUE; i++) {
+ 		}
+ 		alP[i].last = FALSE;
+ 		rows = i+1;
+ 	}
+ 	
+ 	/* Rows required is current rows plus 1 */
+ 	rows++;
+ 
+ 	/* Create enough space for one more item */
+        	alP = (struct check_list_struct *) realloc (alP, (sizeof(struct check_list_struct) * rows));
+ 
+ 	/* Send back the address of this new array */
+ 	*check_listPP = alP;
+ 	
+ 	alP[rows-1].w = w;
+ 	alP[rows-1].c = c;
+ 	alP[rows-1].wpc = wpc;
+ 	alP[rows-1].cpc = cpc;
+ 
+ 	/* Create a buffer big enought for this string and copy it into alp.p */
+ 	buffer_size = sizeof(char) * strlen(p);
+ 	alP[rows-1].p = (char *) malloc(buffer_size);
+ 	strcpy(alP[rows-1].p, p);
+ 	
+ 	alP[rows-1].last = TRUE;
+ 	
+ 	return rows;
+ }
+ 
+ /* Find the thresholds for the file system specified */
+ int check_list_find ( 
+   struct check_list_struct *check_listP, int *wP, int *cP, float *wpcP, float *cpcP, 
+   char *mntP, char *file_system
+ )
+ {
+ 	int i = 0;
+ 	int flag = -1;
+ 	
+ 	/* Set default values if this file system is not found */
+ 	*wP = -1;
+ 	*cP = -1;
+ 	*wpcP = -1,0;
+ 	*cpcP = -1.0;
+  
+ 	if (check_listP == NULL)
+ 		return flag;
+ 
+ 	do {
+ 		if (strcmp(check_listP[i].p,file_system)==0 || strcmp(check_listP[i].p,mntP)==0	) {
+ 			*wP = check_listP[i].w;
+ 			*cP = check_listP[i].c;
+ 			*wpcP = check_listP[i].wpc;
+ 			*cpcP = check_listP[i].cpc;
+ 			flag = 0;
+ 		}
+ 	} while (check_listP[i++].last != TRUE);
+ 
+ 	return flag;	
  }
